#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 ids;
layout(location = 4) in vec4 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	flat ivec4 ids;
	vec4 weights;
	vec4 fragPosLightSpace;
} vert;

uniform mat4 lightSpaceMatrix; //for directional light

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

const int MAX_BONES = 40;
uniform mat4 bones[MAX_BONES];

uniform int renderContour; //0 = false, 1 = true
uniform bool animation;

void main() {
	vec4 posAfterBone = vec4(position, 1.0);
		vert.normal_world = mat3(transpose(inverse(modelMatrix))) * normal;

		if (animation) {
			mat4 boneTransform = bones[ids[0]] *weights[0];
			boneTransform += bones[ids[1]] * weights[1];
			boneTransform += bones[ids[2]] * weights[2];
			boneTransform += bones[ids[3]] * weights[3];
			posAfterBone = boneTransform * vec4(position, 1.0);
			vert.normal_world = mat3(transpose(inverse(boneTransform * modelMatrix))) * normal;
		}
		vert.uv = uv;
		vert.ids = ids;
		vert.weights = weights;
		vec4 position_world_ = modelMatrix * posAfterBone;
		vert.position_world = position_world_.xyz;
		vert.fragPosLightSpace = lightSpaceMatrix * vec4(vert.position_world, 1);
		gl_Position = viewProjMatrix * position_world_;

	
}